local Cursor = require 'cursor'

local CELL_SIZE = 24

Editor = {
  x = 0,
  y = 0,
  interpreter = nil,
  task = nil,
  cursor = nil
}
Editor.__index = Editor;

function Editor.new(interpreter)
  local editor = {}
  setmetatable(editor, Editor)
  editor.interpreter = interpreter
  editor.cursor = Cursor.new(0, 0, interpreter.grid.width, interpreter.grid.height, DIR_RIGHT)
  return editor
end

function Editor.setPosition(x, y)
  self.x = x
  self.y = y
end

function Editor:handleCharKeys(char)
  local code = string.byte(char)
  if code >= 33 and code <= 126 then
    self.interpreter.grid:set(self.cursor.x, self.cursor.y, char)
    self.cursor:move()
  end
end

function Editor:handleControlKeys(key)
  if (key == "left") then
    self.cursor:moveLeft()
  elseif key == "right" then
    self.cursor:moveRight()
  elseif key == "up" then
    self.cursor:moveUp()
  elseif key == "down" then
    self.cursor:moveDown()
  elseif key == "return" then
    self.interpreter:start()
  elseif key == "escape" then
    self.interpreter:stop()
  elseif key == "space" then
    self.interpreter:step()
  end
end

function Editor:drawGrid(w, h)
  for i = 0, w do
    local _x = self.x + i * CELL_SIZE
    local _h = h * CELL_SIZE
    love.graphics.line(_x, self.y, _x, self.y + _h)
  end
  for i = 0, h do
    local _y = self.y + i * CELL_SIZE
    local _w = w * CELL_SIZE
    love.graphics.line(self.x, _y, self.x + _w, _y)
  end
end

function Editor:drawCursor()
  local cursor_c = self.interpreter.run == true and { 255, 0, 0, 64 } or { 255, 255, 255, 64 } -- ternary
  local cursor_x = self.interpreter.run == true and self.interpreter.ip.x or self.cursor.x -- ternary
  local cursor_y = self.interpreter.run == true and self.interpreter.ip.y or self.cursor.y -- ternary
  local draw_x = self.x + cursor_x * CELL_SIZE
  local draw_y = self.y + cursor_y * CELL_SIZE
  love.graphics.setColor(cursor_c)
  love.graphics.rectangle("fill", draw_x, draw_y, CELL_SIZE, CELL_SIZE)
  love.graphics.setColor({255, 255, 255, 255})
end

function Editor:drawValues(w, h)
  for x = 0, w - 1 do
    for y = 0, h - 1 do
      local ch = self.interpreter.grid.cell[x][y]
      local _x = self.x + x * CELL_SIZE
      local _y = self.y + y * CELL_SIZE
      love.graphics.print(ch, _x, _y)
    end
  end
end

function Editor:draw()
  local w = self.interpreter.grid.width
  local h = self.interpreter.grid.height
  
  self:drawGrid(w, h)
  self:drawValues(w, h)
  self:drawCursor()
  -- love.graphics.print("Cursor direction: " .. self.cursor.direction, 400, 0)
  -- love.graphics.print(tostring(self.interpreter.run),  400,  20)
  -- love.graphics.print(self.interpreter.ip.x, 400, 40)
  -- love.graphics.print(self.interpreter.ip.y, 400, 60)
  -- love.graphics.print(self.interpreter.stack:list(), 400, 80)
end

return Editor